//============================================================================
// (c) Copyright 2005 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// DSCDIOFunctions.c  v5.9
// Desc: Writes and reads from Digital I/O port
// Created by KL
//============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "../../../../current/dev/source/dscud.h"
#endif

// Linux and QNX
#if defined(linux) || defined(__QNXNTO__) || defined(_WRS_VXWORKS_5_X)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"


#ifdef _WRS_VXWORKS_5_X
#include <selectLib.h>
#define main PROMDSCDIOFunctions
#else
#include <sys/time.h>
#endif

static int kbhit()
{
	struct timeval timeout;
	fd_set rfds;

	timeout.tv_sec = 0;
	timeout.tv_usec = 0;

	FD_ZERO(&rfds);
	FD_SET(0, &rfds);

	if ( select(0+1, &rfds, NULL, NULL, &timeout) > 0 )
		return getchar();

	return 0;
}
#endif


// var declarations
BYTE result; // returned error code
DSCB dscb;   // handle used to refer to the board
DSCCB dsccb; // structure containing board settings
ERRPARAMS errorParams;  // structure for returning error code and error string
int intBuff;            // temp variables of size int
BYTE config_bytes;      // DIO Configuration byte
BYTE output_b;          // output byte
BYTE input_b;           // input byte
BYTE inputport;            // digital I/O input port
BYTE outputport;           // digital I/O output port
int temp;
int i; // miscellaneous counter
int menuoption;
DSCADSETTINGS adsettings;
DSCSAMPLE sample;
int iterations;

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. DA Conversion
//	    IV. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the PROMETHEUS board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_PROM for PROMETHEUS board
	//	   2. set the base address to 0x280
	//	   3. set the interrupt level 5
	//	   4. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================

	printf( "\nPROM BOARD INITIALIZATION:\n" );

	dsccb.base_address = 0x280;

	dsccb.int_level = 5;

	if(dscInitBoard(DSC_PROM, &dsccb, &dscb)!= DE_NONE)
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// III. DIO INPUT AND OUTPUT
	//
	//		Perform a set of byte-level inputs and outputs. We start by sending
	//		an output byte value of 0 to the selected digital I/O port and then
	//		read from that same port again to verify that the correct value was
	//		exchanged. For word and bit-level operations, the steps are nearly
	//		identical with a few minor changes to function names and the size
	//		of the data to be transferred.
	//
	//		NOTE: Remember that the bytes received from port are supposed to be
	//			  equal to the inversion of the bytes sent. That is, the input
	//			  bytes are equal to 255 minus the output bytes.
	//
	//		STEPS TO FOLLOW:
	//
	//		1. Call dscDIOSetConfig to setup the DIO ports input/output modes. This
	//			maps to base_address + 11 on the Prometheus.
	//		2. Select a port to output on
	//		3. Input the output byte value
	//		4. Select a port for input
	//=========================================================================

	printf( "\nDIO INPUT AND OUTPUT:\n" );

	
	config_bytes = (BYTE) 0x8f;		//set DIO A to out, B and C to in.

	if( (result = dscDIOSetConfig(dscb, &config_bytes)) != DE_NONE) //set DIO configuration
	{
		dscGetLastError(&errorParams);
		fprintf( stderr, "dscDIOSetConfig error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	do
	{
		do
		{
			printf("\nPlease Select from the following options : \n");
			printf("0) Exit\n");
			printf("1) Analog Input\n");
			printf("2) Digital Input\n");
			printf("3) Digital Output\n");

			scanf("%d", &intBuff);
			menuoption = intBuff;

			if( menuoption >= 0 && menuoption <= 3 )
				break;

		} while(1);


		getchar();

		switch(menuoption)
		{
		case 0: dscFree(); return 0;
		case 1: 

			adsettings.gain = 0;
			adsettings.polarity = BIPOLAR;
			adsettings.range = 1;
			adsettings.current_channel = 0;

			result = dscADSetSettings(dscb, &adsettings);
			
			if( result != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "dscADSetSettings error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				break;
			}

			iterations = 0;

			sample = 0;

			printf("\nPerforming A/D Settings, press any key to return to main menu\n");
			printf("   Ch 0      Ch 1      Ch 8      Ch 9      \n");
			printf("===========================================\n");	
			
			
			do{
				
				result = dscADSetChannel(dscb, 0, 0);
				if (result != DE_NONE)
				{
					dscGetLastError(&errorParams);
					fprintf( stderr, "\ndscADSetChannel error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
					break;
				}

				dscADSample(dscb, &sample);
				printf("%7d   ", sample);

				result = dscADSetChannel(dscb, 1, 1);
				if (result != DE_NONE)
				{
					dscGetLastError(&errorParams);
					fprintf( stderr, "\ndscADSetChannel error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
					break;
				}
				dscADSample(dscb, &sample);
				printf("%7d   ", sample);

				result = dscADSetChannel(dscb, 8, 8);
				if (result != DE_NONE)
				{
					dscGetLastError(&errorParams);
					fprintf( stderr, "\ndscADSetChannel error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
					break;
				}
				dscADSample(dscb, &sample);
				printf("%7d   ", sample);

				result = dscADSetChannel(dscb, 9, 9);
				if (result != DE_NONE)
				{
					dscGetLastError(&errorParams);
					fprintf( stderr, "\ndscADSetChannel error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
					break;
				}
				dscADSample(dscb, &sample);
				printf("%7d   ", sample);

				printf("\n");

				iterations ++;
				dscSleep(500);

			}while( !kbhit() );

			if( kbhit() )
				getchar();

			break;

		case 2:
			iterations = 0;

				printf("\nDIO Input, press any key to return to main menu\n");
				printf("DIO B Input    DIO C Input\n");
				printf("==========================\n");
				
			do{
				dscDIOInputByte(dscb, 1, &input_b);
				printf("0x%2x           ", input_b);

				dscDIOInputByte(dscb, 2, &input_b);
				printf("0x%2x   \n", input_b);
				
				iterations ++;
				dscSleep(1000);

			} while(!kbhit());

			if( kbhit() )
				getchar();

			break;

		case 3:

			printf("Please Input the value to output to DIO A in hex : ");
			scanf("%x", &output_b);

			if( ( result = dscDIOOutputByte( dscb, 0, output_b) ) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf( stderr, "\ndscDIOOutputByte error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				break;
			}
			break;
		}

	}while(1);

	

	
	dscFree();

	return 0;
} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
